@@ -36,6 +36,14 @@
 #define WAIT_ANY (-1) /* Any process */
 #endif
 
+#if __BYTE_ORDER == __LITTLE_ENDIAN
+static bool little_endian_system = true;
+#elif __BYTE_ORDER == __BIG_ENDIAN
+static bool little_endian_system = false;
+#else
+#error "Unknown byte order"
+#endif
+
 /**
  * SECTION:test
  * @short_description: Unit test framework
@@ -47,6 +55,7 @@ struct test {
 	const char *name;
 	const void *data;
 	l_test_func_t function;
+	l_test_precheck_t precheck;
 	unsigned long flags;
 	unsigned int num;
 	struct test *next;
@@ -63,12 +72,15 @@ static unsigned int test_count;
 static bool testing_active;
 static int signal_fd;
 
+static bool run_all;
 static bool cmd_list;
 static bool tap_enable;
 static bool debug_enable;
 
 static pid_t test_pid = -1;
 
+static unsigned long default_flags = 0;
+
 /**
  * l_test_init:
  * @argc: pointer to @argc parameter of main() function
@@ -79,6 +91,7 @@ static pid_t test_pid = -1;
 LIB_EXPORT void l_test_init(int *argc, char ***argv)
 {
 	static const struct option options[] = {
+		{ "all",	no_argument,	NULL, 'a' },
 		{ "list",	no_argument,	NULL, 'l' },
 		{ "text",	no_argument,	NULL, 't' },
 		{ "debug",	no_argument,	NULL, 'd' },
@@ -91,6 +104,7 @@ LIB_EXPORT void l_test_init(int *argc, char ***argv)
 
 	l_log_set_stderr();
 
+	run_all = false;
 	cmd_list = false;
 	tap_enable = true;
 	debug_enable = false;
@@ -98,11 +112,14 @@ LIB_EXPORT void l_test_init(int *argc, char ***argv)
 	for (;;) {
 		int opt;
 
-		opt = getopt_long(*argc, *argv, "ltd", options, NULL);
+		opt = getopt_long(*argc, *argv, "altd", options, NULL);
 		if (opt < 0)
 			break;
 
 		switch (opt) {
+		case 'a':
+			run_all = true;
+			break;
 		case 'l':
 			cmd_list = true;
 			break;
@@ -144,7 +161,9 @@ static void print_result(struct test *test, bool success)
 	bool failure_expected = test->flags & L_TEST_FLAG_FAILURE_EXPECTED;
 	bool allow_failure = test->flags & L_TEST_FLAG_ALLOW_FAILURE;
 	bool little_endian = test->flags & L_TEST_FLAG_LITTLE_ENDIAN_ONLY;
+	bool expensive_comp = test->flags & L_TEST_FLAG_EXPENSIVE_COMPUTATION;
 	bool mark_skip = false;
+	const char *comment = NULL;
 
 	if (failure_expected && !success)
 		success = true;
@@ -152,16 +171,27 @@ static void print_result(struct test *test, bool success)
 	if (allow_failure && !success) {
 		success = true;
 		mark_skip = true;
+		comment = " allow-failure";
 	}
 
-	if (little_endian && !success) {
+	if (!little_endian_system && little_endian) {
+		if (!success) {
+			success = true;
+			mark_skip = true;
+		}
+		comment = " little-endian-only";
+	}
+
+	if (expensive_comp && !success) {
 		success = true;
 		mark_skip = true;
+		comment = " expensive-computation";
 	}
 
-	printf("%sok %u - %s%s\n", success ? "" : "not ",
-					test->num, test->name,
-					mark_skip ? " # SKIP" : "");
+	printf("%sok %u - %s%s%s%s\n",
+			success ? "" : "not ", test->num, test->name,
+			(mark_skip || comment) ? " #" : "",
+			mark_skip ? " SKIP" : "", comment ? comment : "");
 }
 
 static void test_setup(struct test *test)
@@ -227,18 +257,12 @@ static void test_sigchld(void *user_data)
 static void dbus_ready(void *user_data)
 {
 	struct test *test = user_data;
-	bool little_endian;
 
-#if __BYTE_ORDER == __LITTLE_ENDIAN
-	little_endian = true;
-#elif __BYTE_ORDER == __BIG_ENDIAN
-	little_endian = false;
-#else
-#error "Unknown byte order"
-#endif
-
-	if ((test->flags & L_TEST_FLAG_LITTLE_ENDIAN_ONLY) && !little_endian) {
-		/* Fail the test case and change the error to SKIP */
+	if (!run_all && (test->flags & L_TEST_FLAG_EXPENSIVE_COMPUTATION)) {
+		/*
+		 * Abort test cases with long running computation task
+		 * to fail and with that be gracefully skipped
+		 */
 		abort();
 		return;
 	}
@@ -291,15 +315,39 @@ static void test_teardown(struct test *test)
 
 static void run_next_test(void *user_data)
 {
+	struct test *test = test_head;
 	pid_t pid;
 
-	if (!test_head) {
+	if (!test) {
 		testing_active = false;
 		return;
 	}
 
 	if (!tap_enable)
-		printf("TEST: %s\n", test_head->name);
+		printf("TEST: %s\n", test->name);
+
+	if (test->precheck) {
+		bool result = test->precheck(test->data);
+
+		if (test->flags & L_TEST_FLAG_INVERT_PRECHECK_RESULT)
+			result = !result;
+
+		if (!result) {
+			if (tap_enable)
+				printf("ok %u - %s # SKIP not-supported\n",
+							test->num, test->name);
+
+			test_head = test->next;
+			free(test);
+
+			if (!test_head)
+				test_tail = NULL;
+
+			/* Trigger the main pollfd loop */
+			kill(getpid(), SIGUSR1);
+			return;
+		}
+	}
 
 	pid = fork();
 	if (pid < 0) {
@@ -318,9 +366,9 @@ static void run_next_test(void *user_data)
 		/* Close stdout to not interfere with TAP */
 		close(STDOUT_FILENO);
 
-		test_setup(test_head);
-		test_function(test_head);
-		test_teardown(test_head);
+		test_setup(test);
+		test_function(test);
+		test_teardown(test);
 
 		exit(EXIT_SUCCESS);
 	}
@@ -416,6 +464,7 @@ LIB_EXPORT int l_test_run(void)
 	sigaddset(&sig_mask, SIGINT);
 	sigaddset(&sig_mask, SIGTERM);
 	sigaddset(&sig_mask, SIGCHLD);
+	sigaddset(&sig_mask, SIGUSR1);
 
 	/*
 	 * Block signals so that they aren't handled according to their
@@ -466,6 +515,9 @@ LIB_EXPORT int l_test_run(void)
 		case SIGCHLD:
 			sigchld_handler(NULL);
 			break;
+		case SIGUSR1:
+			run_next_test(NULL);
+			break;
 		}
 	}
 
@@ -476,32 +528,10 @@ LIB_EXPORT int l_test_run(void)
 	return exit_status;
 }
 
-/**
- * l_test_add_data_func:
- * @name: test name
- * @function: test function
- * @flags: test flags;
- *
- * Add new test.
- **/
-LIB_EXPORT void l_test_add_func(const char *name, l_test_func_t function,
-							unsigned long flags)
-{
-	l_test_add_data_func(name, NULL, function, flags);
-}
-
-/**
- * l_test_add_data_func:
- * @name: test name
- * @function: test function
- * @data: test data
- * @flags: test flags;
- *
- * Add new test.
- **/
-LIB_EXPORT void l_test_add_data_func(const char *name, const void *data,
-							l_test_func_t function,
-							unsigned long flags)
+static void common_add(const char *name, const void *data,
+						l_test_func_t function,
+						l_test_precheck_t precheck,
+						unsigned long flags)
 {
 	struct test *test;
 
@@ -516,6 +546,7 @@ LIB_EXPORT void l_test_add_data_func(const char *name, const void *data,
 	test->name = name;
 	test->data = data;
 	test->function = function;
+	test->precheck = precheck;
 	test->flags = flags;
 	test->num = ++test_count;
 	test->next = NULL;
@@ -529,6 +560,73 @@ LIB_EXPORT void l_test_add_data_func(const char *name, const void *data,
 		test_head = test;
 }
 
+/**
+ * l_test_add_func_precheck:
+ * @name: test name
+ * @data: test data
+ * @function: test function
+ * @precheck: precheck function
+ * @flags: test flags;
+ *
+ * Add new test.
+ **/
+LIB_EXPORT void l_test_add_func_precheck(const char *name,
+						l_test_func_t function,
+						l_test_precheck_t precheck,
+						unsigned long flags)
+{
+	common_add(name, NULL, function, precheck, flags);
+}
+
+/**
+ * l_test_add_data_func_precheck:
+ * @name: test name
+ * @data: test data
+ * @function: test function
+ * @precheck: precheck function
+ * @flags: test flags;
+ *
+ * Add new test.
+ **/
+LIB_EXPORT void l_test_add_data_func_precheck(const char *name,
+						const void *data,
+						l_test_func_t function,
+						l_test_precheck_t precheck,
+						unsigned long flags)
+{
+	common_add(name, data, function, precheck, flags);
+}
+
+/**
+ * l_test_add_data_func:
+ * @name: test name
+ * @function: test function
+ * @flags: test flags;
+ *
+ * Add new test.
+ **/
+LIB_EXPORT void l_test_add_func(const char *name, l_test_func_t function,
+							unsigned long flags)
+{
+	common_add(name, NULL, function, NULL, flags);
+}
+
+/**
+ * l_test_add_data_func:
+ * @name: test name
+ * @data: test data
+ * @function: test function
+ * @flags: test flags;
+ *
+ * Add new test.
+ **/
+LIB_EXPORT void l_test_add_data_func(const char *name, const void *data,
+							l_test_func_t function,
+							unsigned long flags)
+{
+	common_add(name, data, function, NULL, flags);
+}
+
 /**
  * l_test_add:
  * @name: test name
@@ -540,5 +638,16 @@ LIB_EXPORT void l_test_add_data_func(const char *name, const void *data,
 LIB_EXPORT void l_test_add(const char *name, l_test_func_t function,
 							const void *data)
 {
-	l_test_add_data_func(name, data, function, 0);
+	common_add(name, data, function, NULL, default_flags);
+}
+
+/**
+ * l_test_set_default_flags:
+ * @flags: test flags;
+ *
+ * Set default for test flags.
+ **/
+LIB_EXPORT void l_test_set_default_flags(unsigned long flags)
+{
+	default_flags = flags;
 }
